@@ -10,12 +10,12 @@
 #include "config_handler.h"
 #include "globalscalesettings.h"
 #include "kcm_screen_debug.h"
-#include "output_model.h"
 
 #include <kscreen/config.h>
 #include <kscreen/configmonitor.h>
 #include <kscreen/getconfigoperation.h>
 #include <kscreen/log.h>
+#include <kscreen/mode.h>
 #include <kscreen/output.h>
 #include <kscreen/setconfigoperation.h>
 
@@ -39,6 +39,7 @@ KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data, const QVari
     qmlRegisterAnonymousType<OutputModel>("org.kde.private.kcm.screen", 1);
     qmlRegisterType<KScreen::Output>("org.kde.private.kcm.kscreen", 1, 0, "Output");
     qmlRegisterUncreatableType<Control>("org.kde.private.kcm.kscreen", 1, 0, "Control", QStringLiteral("Provides only the OutputRetention enum class"));
+    qmlRegisterUncreatableType<KCMKScreen>("org.kde.private.kcm.kscreen", 1, 0, "KCMKScreen", QStringLiteral("For InvalidConfig enum"));
     Log::instance();
 
     setButtons(Apply);
@@ -74,6 +75,7 @@ void KCMKScreen::configReady(ConfigOperation *op)
 
     m_configHandler->setConfig(config);
     setBackendReady(true);
+    checkConfig();
     Q_EMIT perOutputScalingChanged();
     Q_EMIT xwaylandClientsScaleSupportedChanged();
     Q_EMIT primaryOutputSupportedChanged();
@@ -82,25 +84,20 @@ void KCMKScreen::configReady(ConfigOperation *op)
     Q_EMIT autoRotationSupportedChanged();
 }
 
-void KCMKScreen::forceSave()
-{
-    doSave(true);
-}
-
 void KCMKScreen::save()
 {
-    doSave(false);
+    doSave();
 }
 
 void KCMKScreen::revertSettings()
 {
-    if (!m_configHandler) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return;
     }
     if (!m_settingsReverted) {
         m_configHandler->revertConfig();
         m_settingsReverted = true;
-        doSave(true);
+        doSave();
         load(); // reload the configuration
         Q_EMIT settingsReverted();
         m_stopUpdatesFromBackend = false;
@@ -130,39 +127,29 @@ void KCMKScreen::updateFromBackend()
     m_loadCompressor->start();
 }
 
-void KCMKScreen::doSave(bool force)
+void KCMKScreen::doSave()
 {
-    if (!m_configHandler) {
+    if (!m_configHandler || !m_configHandler->config()) {
         Q_EMIT errorOnSave();
         return;
     }
 
-    auto config = m_configHandler->config();
-
-    bool atLeastOneEnabledOutput = false;
-    const auto outputs = config->outputs();
+    const auto outputs = m_configHandler->config()->outputs();
     for (const KScreen::OutputPtr &output : outputs) {
         KScreen::ModePtr mode = output->currentMode();
-
-        atLeastOneEnabledOutput |= output->isEnabled();
-
         qCDebug(KSCREEN_KCM) << output->name() << output->id() << output.data() << "\n"
-                             << "	Connected:" << output->isConnected() << "\n"
-                             << "	Enabled:" << output->isEnabled() << "\n"
-                             << "	Primary:" << output->isPrimary() << "\n"
-                             << "	Rotation:" << output->rotation() << "\n"
-                             << "	Mode:" << (mode ? mode->name() : QStringLiteral("unknown")) << "@" << (mode ? mode->refreshRate() : 0.0) << "Hz"
+                             << "\tConnected:" << output->isConnected() << "\n"
+                             << "\tEnabled:" << output->isEnabled() << "\n"
+                             << "\tPriority:" << output->priority() << "\n"
+                             << "\tRotation:" << output->rotation() << "\n"
+                             << "\tMode:" << (mode ? mode->name() : QStringLiteral("unknown")) << "@" << (mode ? mode->refreshRate() : 0.0) << "Hz"
                              << "\n"
-                             << "    Position:" << output->pos().x() << "x" << output->pos().y() << "\n"
-                             << "    Scale:" << (perOutputScaling() ? QString::number(output->scale()) : QStringLiteral("global")) << "\n"
-                             << "    Replicates:" << (output->replicationSource() == 0 ? "no" : "yes");
+                             << "\tPosition:" << output->pos().x() << "x" << output->pos().y() << "\n"
+                             << "\tScale:" << (perOutputScaling() ? QString::number(output->scale()) : QStringLiteral("global")) << "\n"
+                             << "\tReplicates:" << (output->replicationSource() == 0 ? "no" : "yes");
     }
 
-    if (!atLeastOneEnabledOutput && !force) {
-        Q_EMIT dangerousSave();
-        m_configHandler->checkNeedsSave();
-        return;
-    }
+    auto config = m_configHandler->config();
 
     if (!Config::canBeApplied(config)) {
         Q_EMIT errorOnSave();
@@ -186,7 +173,7 @@ void KCMKScreen::doSave(bool force)
     op->exec();
 
     const auto updateInitialData = [this]() {
-        if (!m_configHandler) {
+        if (!m_configHandler || !m_configHandler->config()) {
             return;
         }
         m_configHandler->updateInitialData();
@@ -360,9 +347,38 @@ void KCMKScreen::load()
     Q_EMIT changed();
 }
 
+void KCMKScreen::checkConfig()
+{
+    const auto outputs = m_configHandler->config()->outputs();
+    std::vector<OutputPtr> enabledOutputs;
+    std::copy_if(outputs.cbegin(), outputs.cend(), std::back_inserter(enabledOutputs), std::mem_fn(&Output::isEnabled));
+    if (enabledOutputs.empty()) {
+        Q_EMIT invalidConfig(NoEnabledOutputs);
+        m_configNeedsSave = false;
+    }
+    auto rectsTouch = [](const QRect &rect, const QRect &other) {
+        return rect.left() <= other.left() + other.width() && other.left() <= rect.left() + rect.width() && rect.top() <= other.top() + other.height()
+            && other.top() <= rect.top() + rect.height();
+    };
+    auto doesNotTouchAnyOther = [&enabledOutputs, &rectsTouch](const OutputPtr &output) {
+        return std::none_of(enabledOutputs.cbegin(), enabledOutputs.cend(), [&output, &rectsTouch](const OutputPtr &other) {
+            return other != output && rectsTouch(output->geometry(), other->geometry());
+        });
+    };
+    if (enabledOutputs.size() > 1 && std::any_of(enabledOutputs.cbegin(), enabledOutputs.cend(), doesNotTouchAnyOther)) {
+        Q_EMIT invalidConfig(ConfigHasGaps);
+        m_configNeedsSave = false;
+    }
+}
+
 void KCMKScreen::continueNeedsSaveCheck(bool needs)
 {
     m_configNeedsSave = needs;
+
+    if (needs) {
+        checkConfig();
+    }
+
     settingsChanged();
 }
 
